"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exclusiveProps = void 0;
/* eslint-disable react/no-array-index-key */
const react_1 = __importStar(require("react"));
const hooks_1 = require("../hooks");
function exclusiveProps(margins, a, b, position) {
    if (margins[a] !== undefined) {
        if (margins[b] !== undefined) {
            throw new Error(`${a} and ${b} should't be both defined for the position ${position}`);
        }
        return { key: a, value: margins[a] };
    }
    return margins[b] !== undefined ? { key: b, value: margins[b] } : {};
}
exports.exclusiveProps = exclusiveProps;
function translation(position, legendMargins, plotMargins, width, height) {
    const plotHeight = height - plotMargins.top - plotMargins.bottom;
    const plotWidth = width - plotMargins.left - plotMargins.right;
    switch (position) {
        case 'embedded': {
            const { key: verticalKey = 'top', value: verticalValue = 10, } = exclusiveProps(legendMargins, 'top', 'bottom', position);
            const { key: horizontalKey = 'left', value: horizontalValue = 10, } = exclusiveProps(legendMargins, 'left', 'right', position);
            const x = horizontalKey === 'right'
                ? width - plotMargins.right - horizontalValue
                : plotMargins.left + horizontalValue;
            const y = verticalKey === 'bottom'
                ? height - plotMargins.bottom - verticalValue
                : plotMargins.top + verticalValue;
            return [x, y];
        }
        case 'top': {
            const { key: horizontalKey = 'left', value: horizontalValue = plotWidth / 2, } = exclusiveProps(legendMargins, 'left', 'right', position);
            const x = horizontalKey === 'right'
                ? width - plotMargins.right + horizontalValue
                : plotMargins.left + horizontalValue;
            const y = plotMargins.top - (legendMargins.bottom || 50);
            return [x, y];
        }
        case 'bottom': {
            const { key: horizontalKey = 'left', value: horizontalValue = plotWidth / 2, } = exclusiveProps(legendMargins, 'left', 'right', position);
            const x = horizontalKey === 'right'
                ? width - plotMargins.right + horizontalValue
                : plotMargins.left + horizontalValue;
            const y = height - plotMargins.bottom + (legendMargins.top || 25);
            return [x, y];
        }
        case 'left': {
            const { key: verticalKey = 'top', value: verticalValue = plotHeight / 2, } = exclusiveProps(legendMargins, 'top', 'bottom', position);
            const y = verticalKey === 'bottom'
                ? height - plotMargins.bottom - verticalValue
                : plotMargins.top + verticalValue;
            const x = plotMargins.left - (legendMargins.right || 100);
            return [x, y];
        }
        case 'right': {
            const { key: verticalKey = 'top', value: verticalValue = plotHeight / 2, } = exclusiveProps(legendMargins, 'top', 'bottom', position);
            const y = verticalKey === 'bottom'
                ? height - plotMargins.bottom - verticalValue
                : plotMargins.top + verticalValue;
            const x = width - plotMargins.right + (legendMargins.left || 40);
            return [x, y];
        }
        default: {
            throw new Error(`Position ${JSON.stringify(position)} unknown`);
        }
    }
}
function Legend({ position, ...legendMargins }) {
    const { labels, right, left, top, bottom, height, width, colorScaler, } = hooks_1.usePlotContext();
    const [x, y] = react_1.useMemo(() => {
        const plotMargins = { right, left, top, bottom };
        return translation(position, legendMargins, plotMargins, width, height);
    }, [position, legendMargins, right, left, top, bottom, width, height]);
    return (react_1.default.createElement("g", { className: "legend", transform: `translate(${x}, ${y})` }, labels === null || labels === void 0 ? void 0 :
        labels.map(({ id }, index) => (react_1.default.createElement("circle", { key: `circle-${id}-${index}`, cx: "0.25em", cy: `${index + 0.7}em`, r: "0.25em", fill: colorScaler(id) }))), labels === null || labels === void 0 ? void 0 :
        labels.map(({ label }, index) => (react_1.default.createElement("text", { key: `text-${label}-${index}`, x: "0.75em", y: `${index + 1}em` }, label)))));
}
exports.default = Legend;
