"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.functionalStyle = exports.validateAxis = exports.validatePosition = exports.splitChildren = exports.getNextId = void 0;
const react_1 = __importDefault(require("react"));
const Axis_1 = __importDefault(require("./components/Axis"));
const Heading_1 = __importDefault(require("./components/Heading"));
const Legend_1 = __importDefault(require("./components/Legend"));
const LineSeries_1 = __importDefault(require("./components/LineSeries"));
const ScatterSeries_1 = __importDefault(require("./components/ScatterSeries"));
let currentValue = 1;
/**
 * Creates autoincremental values, generally for series
 */
function getNextId() {
    return ++currentValue;
}
exports.getNextId = getNextId;
/**
 * Validates that all the items inside Plot are supported and organizes by kind
 */
function splitChildren(children) {
    let invalidChild = false;
    let series = [];
    let axis = [];
    let heading = null;
    let legend = null;
    for (let child of react_1.default.Children.toArray(children)) {
        // Base child validation
        if (typeof child !== 'object' || !react_1.default.isValidElement(child)) {
            invalidChild = true;
        }
        // Classifies the expected components
        else if (child.type === LineSeries_1.default || child.type === ScatterSeries_1.default) {
            series.push(child);
        }
        else if (child.type === Axis_1.default) {
            axis.push(child);
        }
        else if (child.type === Heading_1.default) {
            heading = child;
        }
        else if (child.type === Legend_1.default) {
            legend = child;
        }
        // Default case
        else {
            invalidChild = true;
        }
    }
    return { invalidChild, series, axis, heading, legend };
}
exports.splitChildren = splitChildren;
const horizontal = ['top', 'bottom'];
const vertical = ['left', 'right'];
/**
 * Validates that two positions are not orthogonal between them
 */
function validatePosition(currPosition, position, id) {
    const error = (horizontal.includes(currPosition) && !horizontal.includes(position)) ||
        (vertical.includes(currPosition) && !vertical.includes(position));
    if (error) {
        throw new Error(`The positions are ortogonal for ${id}`);
    }
}
exports.validatePosition = validatePosition;
/**
 * Validates that two axes are orthogonal between them
 */
function validateAxis(axisContext, xKey, yKey) {
    const xAxis = axisContext[xKey];
    const yAxis = axisContext[yKey];
    if (!xAxis || !yAxis)
        return [undefined, undefined];
    if (horizontal.includes(xAxis.position)
        ? !vertical.includes(yAxis.position)
        : vertical.includes(xAxis.position)) {
        throw new Error(`The axis ${xKey} and ${yKey} are not orthogonal`);
    }
    return [xAxis.scale, yAxis.scale];
}
exports.validateAxis = validateAxis;
/**
 * Checks the style added to a component and if is a function, gets the resulting value
 */
function functionalStyle(defaultStyle, elementStyle, point, index, data) {
    let style = { ...defaultStyle };
    for (const key in elementStyle) {
        if (typeof elementStyle[key] === 'function') {
            style[key] = elementStyle[key](point, index, data);
        }
        else {
            style[key] = elementStyle[key];
        }
    }
    return style;
}
exports.functionalStyle = functionalStyle;
